@@ -4,6 +4,7 @@
 #include "hash_print.h"
 #include "output.h"
 #include "parse_cmdline.h"
+#include "platform.h"
 #include "rhash_main.h"
 #include "win_utils.h"
 #include "librhash/rhash.h"
@@ -18,6 +19,107 @@
 # include <io.h>
 #endif
 
+
+/*=========================================================================
+ * Hash identifiers functions
+ *=========================================================================*/
+
+/**
+ * Convert 64-bit hash_mask to a sequence of hash function identifiers.
+ *
+ * @param hash_mask bit mask
+ * @param max_count maximum space in the hash_ids array
+ * @param hash_ids array to store sequence of hash function identifiers
+ * @param out_count count of stored hash function identifiers
+ * @return -1 on error, 0 otherwise
+ */
+int hash_mask_to_hash_ids(uint64_t hash_mask, unsigned max_count,
+	unsigned* hash_ids, unsigned* out_count)
+{
+	unsigned count;
+	if (!hash_ids || !out_count)
+		return -1;
+	for (count = 0; hash_mask; count++) {
+		uint64_t bit64 = hash_mask & -hash_mask;
+		if (count == max_count)
+			return -1;
+		hash_ids[count] = bit64_to_hash_id(bit64);
+		hash_mask ^= bit64;
+	}
+	*out_count = count;
+	return 0;
+}
+
+static uint64_t hash_ids_to_hash_mask(size_t count, unsigned* hash_ids)
+{
+	uint64_t hash_mask = 0;
+	size_t i;
+	for (i = 0; i < count; i++)
+		hash_mask |= hash_id_to_bit64(hash_ids[i]);
+	return hash_mask;
+}
+
+/**
+ * Enable openssl implementation for algorithms selected by the hash_mask.
+ *
+ * @param hash_mask bit mask for enabled hash functions
+ * @return -1 on error, 0 otherwise
+ */
+int set_openssl_enabled_hash_mask(uint64_t hash_mask)
+{
+	unsigned hash_ids[64];
+	unsigned count;
+	size_t res;
+	hash_mask &= ~OPENSSL_MASK_VALID_BIT; /* remove validity bit */
+	if (hash_mask_to_hash_ids(hash_mask, 64, hash_ids, &count) < 0)
+		return -1;
+	rhash_set_openssl_enabled(count, hash_ids);
+	res = rhash_get_openssl_enabled(0, NULL);
+	return (res != RHASH_ERROR ? (int)res : -1);
+}
+
+#define unknown_bit 0x8000000000000000
+
+/**
+ * Return hash_mask for algorithms supported by openssl.
+ ^
+ * @return bit mask for supported hash functions
+ */
+uint64_t get_openssl_supported_hash_mask(void)
+{
+	static uint64_t supported = unknown_bit;
+	if (supported == unknown_bit) {
+		unsigned hash_ids[64];
+		size_t count = rhash_get_openssl_supported(64, hash_ids);
+		if (count != RHASH_ERROR)
+			supported = hash_ids_to_hash_mask(count, hash_ids);
+		supported &= ~unknown_bit;
+	}
+	return supported;
+}
+
+/**
+ * Return hash_mask for algorithms supported by openssl.
+ ^
+ * @return bit mask for supported hash functions
+ */
+uint64_t get_all_supported_hash_mask(void)
+{
+	static uint64_t hash_mask = unknown_bit;
+	if (hash_mask == unknown_bit) {
+		unsigned hash_ids[64];
+		size_t count = rhash_get_all_algorithms(64, hash_ids);
+		RSH_REQUIRE(count != RHASH_ERROR, "failed to get all supported algorithms\n");
+		hash_mask = hash_ids_to_hash_mask(count, hash_ids);
+		hash_mask &= ~unknown_bit;
+	}
+	return hash_mask;
+}
+
+/*=========================================================================
+ * Hash calculation functions
+ *=========================================================================*/
+
 /**
  * Initialize BTIH hash function. Unlike other algorithms BTIH
  * requires more data for correct computation.
@@ -26,7 +128,7 @@
  */
 static void init_btih_data(struct file_info* info)
 {
-	assert((info->rctx->hash_id & RHASH_BTIH) != 0);
+	assert((info->rctx->hash_mask & RHASH_BTIH) != 0);
 
 	if (opt.flags & (OPT_BT_PRIVATE | OPT_BT_TRANSMISSION)) {
 		unsigned options = (opt.flags & OPT_BT_PRIVATE ? RHASH_TORRENT_OPT_PRIVATE : 0)
@@ -60,8 +162,8 @@ static void init_btih_data(struct file_info* info)
 static void re_init_rhash_context(struct file_info* info)
 {
 	if (rhash_data.rctx != 0) {
-		if (IS_MODE(MODE_CHECK | MODE_CHECK_EMBEDDED)) {
-			/* a set of hash algorithms can change from file to file */
+		if (IS_MODE(MODE_CHECK | MODE_CHECK_EMBEDDED) && rhash_data.last_hash_mask != info->hash_mask) {
+			/* a set of hash algorithms has changed from the previous run */
 			rhash_free(rhash_data.rctx);
 			rhash_data.rctx = 0;
 		} else {
@@ -78,18 +180,27 @@ static void re_init_rhash_context(struct file_info* info)
 	}
 
 	if (rhash_data.rctx == 0) {
-		rhash_data.rctx = rhash_init(info->sums_flags);
+		uint64_t hash_mask = info->hash_mask;
+		if (rhash_data.last_hash_mask != hash_mask) {
+			unsigned count = 0;
+			RSH_REQUIRE(hash_mask_to_hash_ids(hash_mask, 64, rhash_data.hash_ids, &count) >= 0,
+				"failed to convert hash ids\n");
+			rhash_data.hash_ids_count = count;
+			rhash_data.last_hash_mask = hash_mask;
+		}
+		rhash_data.rctx = rhash_init_multi(rhash_data.hash_ids_count, rhash_data.hash_ids);
 		info->rctx = rhash_data.rctx;
+		RSH_REQUIRE(rhash_data.rctx, "failed to initialize hash context\n");
 	}
 
-	if (info->sums_flags & RHASH_BTIH) {
+	if (info->hash_mask & hash_id_to_bit64(RHASH_BTIH)) {
 		/* re-initialize BitTorrent data */
 		init_btih_data(info);
 	}
 }
 
 /**
- * Calculate message digests simultaneously, according to the info->sums_flags.
+ * Calculate message digests simultaneously, according to the info->hash_mask.
  * Calculated message digests are stored in info->rctx.
  *
  * @param info file data
@@ -97,15 +208,15 @@ static void re_init_rhash_context(struct file_info* info)
  */
 int calc_sums(struct file_info* info)
 {
-	FILE* fd = 0;
+	int fd = -1;
 	int res;
 
 	assert(info->file);
 	if (FILE_ISSTDIN(info->file)) {
-		fd = stdin;
+		fd = 0;
 #ifdef _WIN32
 		/* using 0 instead of _fileno(stdin). _fileno() is undefined under 'gcc -ansi' */
-		if (setmode(0, _O_BINARY) < 0)
+		if (setmode(fd, _O_BINARY) < 0)
 			return -1;
 #endif
 	} else {
@@ -116,13 +227,13 @@ int calc_sums(struct file_info* info)
 
 		info->size = info->file->size; /* total size, in bytes */
 
-		if (!info->sums_flags)
+		if (!info->hash_mask)
 			return 0;
 
 		if (!FILE_ISDATA(info->file)) {
-			fd = file_fopen(info->file, FOpenRead | FOpenBin);
+			fd = file_open(info->file, FOpenReadBin);
 			/* quietly skip unreadble files */
-			if (!fd)
+			if (fd < 0)
 				return -1;
 		}
 	}
@@ -138,7 +249,7 @@ int calc_sums(struct file_info* info)
 		if (percents_output->update != 0) {
 			rhash_set_callback(info->rctx, (rhash_callback_t)percents_output->update, info);
 		}
-		res = rhash_file_update(info->rctx, fd);
+		res = rhash_update_fd(info->rctx, fd, RHASH_MAX_FILE_SIZE);
 	}
 	if (res != -1 && !opt.bt_batch_file)
 		rhash_final(info->rctx, 0); /* finalize hashing */
@@ -147,8 +258,8 @@ int calc_sums(struct file_info* info)
 	info->size = info->rctx->msg_size - info->msg_offset;
 	rhash_data.total_size += info->size;
 
-	if (fd && !FILE_ISSTDIN(info->file))
-		fclose(fd);
+	if (fd >= 0 && !FILE_ISSTDIN(info->file))
+		close(fd);
 	return res;
 }
 
@@ -202,7 +313,7 @@ int rename_file_by_embeding_crc32(struct file_info* info)
 	if (FILE_ISSPECIAL(info->file))
 		return 0; /* do nothing on stdin or a command-line message */
 
-	assert((info->rctx->hash_id & RHASH_CRC32) != 0);
+	assert((info->rctx->hash_mask & RHASH_CRC32) != 0);
 	rhash_print(suffix + 2, info->rctx, RHASH_CRC32,
 		(opt.flags & OPT_LOWERCASE ? 0 : RHPR_UPPERCASE));
 
@@ -322,7 +433,7 @@ int calculate_and_print_sums(FILE* out, file_t* out_file, file_t* file)
 	memset(&info, 0, sizeof(info));
 	info.file = file;
 	info.size = file->size; /* total size, in bytes */
-	info.sums_flags = opt.sum_flags;
+	info.hash_mask = opt.hash_mask;
 
 	/* initialize percents output */
 	if (init_percents(&info) < 0) {
@@ -331,7 +442,8 @@ int calculate_and_print_sums(FILE* out, file_t* out_file, file_t* file)
 	}
 	rsh_timer_start(&timer);
 
-	if (info.sums_flags) {
+	if (info.hash_mask) {
+		print_verbose_algorithms(rhash_data.log, info.hash_mask);
 		/* calculate sums */
 		if (calc_sums(&info) < 0) {
 			/* print i/o error */
@@ -363,7 +475,7 @@ int calculate_and_print_sums(FILE* out, file_t* out_file, file_t* file)
 			log_error_file_t(out_file);
 			res = -2;
 		}
-		if (opt.flags & OPT_VERBOSE) {
+		if (opt.verbose) {
 			print_sfv_header_line(rhash_data.log, rhash_data.log_file.mode, file);
 			fflush(rhash_data.log);
 		}
@@ -376,12 +488,12 @@ int calculate_and_print_sums(FILE* out, file_t* out_file, file_t* file)
 				res = -2;
 			}
 			/* print the calculated line to stderr/log-file if verbose */
-			else if (IS_MODE(MODE_UPDATE) && (opt.flags & OPT_VERBOSE)) {
+			else if (IS_MODE(MODE_UPDATE) && opt.verbose) {
 				print_line(rhash_data.log, rhash_data.log_file.mode, rhash_data.print_list, &info);
 			}
 		}
 
-		if ((opt.flags & OPT_SPEED) && info.sums_flags) {
+		if ((opt.flags & OPT_SPEED) && info.hash_mask) {
 			print_file_time_stats(&info);
 		}
 	}
@@ -393,28 +505,30 @@ int calculate_and_print_sums(FILE* out, file_t* out_file, file_t* file)
  *=========================================================================*/
 
 /**
- * Hash a repeated message chunk by specified hash function.
+ * Calculate message digest of specified hash function(s) for a repeated message chunk.
  *
- * @param hash_id hash function identifier
+ * @param hash_ids_count count of hash functions
+ * @param hash_ids array hash function identifiers
  * @param message a message chunk to hash
  * @param msg_size message chunk size
  * @param count number of chunks
  * @param out computed hash
  * @return 1 on success, 0 on error
  */
-static int benchmark_loop(unsigned hash_id, const unsigned char* message, size_t msg_size, int count, unsigned char* out)
+static int benchmark_loop(unsigned hash_ids_count, unsigned hash_ids[],
+	const unsigned char* message, size_t msg_size, int count, unsigned char* out)
 {
 	int i;
-	struct rhash_context* context = rhash_init(hash_id);
-	if (!context)
+	rhash ctx = rhash_init_multi(hash_ids_count, hash_ids);
+	if (!ctx)
 		return 0;
 
 	/* process the repeated message buffer */
 	for (i = 0; i < count && !rhash_data.stop_flags; i++) {
-		rhash_update(context, message, msg_size);
+		rhash_update(ctx, message, msg_size);
 	}
-	rhash_final(context, out);
-	rhash_free(context);
+	rhash_final(ctx, out);
+	rhash_free(ctx);
 	return 1;
 }
 
@@ -445,7 +559,7 @@ static uint64_t read_tsc(void) {
 #endif /* _MSC_VER, __GNUC__ */
 #endif /* x86/amd64 arch */
 
-void run_benchmark(unsigned hash_id, unsigned flags)
+void run_benchmark(uint64_t hash_mask, unsigned flags)
 {
 	unsigned char ALIGN_DATA(64) message[8192]; /* 8 KiB */
 	timedelta_t timer;
@@ -453,7 +567,9 @@ void run_benchmark(unsigned hash_id, unsigned flags)
 	size_t sz_mb, msg_size;
 	uint64_t time, total_time = 0;
 	const int rounds = 4;
-	const char* hash_name;
+	const char* hash_name = "";
+	unsigned hash_ids[64];
+	unsigned hash_ids_count = 0;
 	unsigned char out[130];
 #ifdef HAVE_TSC
 	double cpb = 0;
@@ -462,25 +578,32 @@ void run_benchmark(unsigned hash_id, unsigned flags)
 #ifdef _WIN32
 	set_benchmark_cpu_affinity(); /* set CPU affinity to improve test results */
 #endif
+	if (!(flags & BENCHMARK_RAW))
+		print_verbose_algorithms(rhash_data.out, hash_mask);
 
 	/* set message size for fast and slow hash functions */
 	msg_size = 1073741824 / 2;
-	if (hash_id & (RHASH_WHIRLPOOL | RHASH_SNEFRU128 | RHASH_SNEFRU256 | RHASH_SHA3_224 | RHASH_SHA3_256 | RHASH_SHA3_384 | RHASH_SHA3_512)) {
+	if (hash_mask & (RHASH_WHIRLPOOL | RHASH_SNEFRU128 | RHASH_SNEFRU256 | RHASH_SHA3_224 | RHASH_SHA3_256 | RHASH_SHA3_384 | RHASH_SHA3_512)) {
 		msg_size /= 8;
-	} else if (hash_id & (RHASH_GOST94 | RHASH_GOST94_CRYPTOPRO | RHASH_SHA384 | RHASH_SHA512)) {
+	} else if (hash_mask & (RHASH_GOST94 | RHASH_GOST94_CRYPTOPRO | RHASH_SHA384 | RHASH_SHA512)) {
 		msg_size /= 2;
 	}
 	sz_mb = msg_size / (1 << 20); /* size in MiB */
-	hash_name = rhash_get_name(hash_id);
-	if (!hash_name)
-		hash_name = ""; /* benchmarking several hash functions */
+
+	if (hash_mask && (hash_mask & (hash_mask - 1)) == 0) {
+		hash_name = rhash_get_name(bit64_to_hash_id(hash_mask));
+		if (!hash_name) hash_name = ""; /* unsupported hash function */
+	}
+	RSH_REQUIRE(hash_mask_to_hash_ids(hash_mask, 64, hash_ids, &hash_ids_count) >= 0,
+		"failed to convert hash ids\n");
 
 	for (i = 0; i < (int)sizeof(message); i++)
 		message[i] = i & 0xff;
 
 	for (j = 0; j < rounds && !rhash_data.stop_flags; j++) {
 		rsh_timer_start(&timer);
-		benchmark_loop(hash_id, message, sizeof(message), (int)(msg_size / sizeof(message)), out);
+		benchmark_loop(hash_ids_count, hash_ids, message, sizeof(message),
+			(int)(msg_size / sizeof(message)), out);
 
 		time = rsh_timer_stop(&timer);
 		total_time += time;
@@ -502,10 +625,10 @@ void run_benchmark(unsigned hash_id, unsigned flags)
 		/* make 200 tries */
 		for (i = 0; i < 200; i++) {
 			cy0 = read_tsc();
-			benchmark_loop(hash_id, message, sizeof(message), msg_size / sizeof(message), out);
+			benchmark_loop(hash_ids_count, hash_ids, message, sizeof(message), msg_size / sizeof(message), out);
 			cy1 = read_tsc();
-			benchmark_loop(hash_id, message, sizeof(message), msg_size / sizeof(message), out);
-			benchmark_loop(hash_id, message, sizeof(message), msg_size / sizeof(message), out);
+			benchmark_loop(hash_ids_count, hash_ids, message, sizeof(message), msg_size / sizeof(message), out);
+			benchmark_loop(hash_ids_count, hash_ids, message, sizeof(message), msg_size / sizeof(message), out);
 			cy2 = read_tsc();
 
 			cy2 -= cy1;
