@@ -1,25 +1,22 @@
-/* common_func.c */
+/* common_func.c - functions used almost everywhere */
 
-#include "common_func.h" /* should be included before the C library files */
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-#include <assert.h>
-#include <stdint.h>
-#include <time.h>
-#include <stdarg.h>
-#include <wchar.h>
-#include <errno.h>
-#include <sys/stat.h>
-
-#ifdef _WIN32
-#include <windows.h>
-#endif
-
-#include "win_utils.h"
+#include "common_func.h"
+#include "output.h"
 #include "parse_cmdline.h"
 #include "version.h"
+#include "win_utils.h"
+#include <assert.h>
+#include <ctype.h>
+#include <errno.h>
+#include <stdarg.h>
+#include <stdlib.h>
+#include <string.h>
+#include <wchar.h>
+
+#if defined( _WIN32) || defined(__CYGWIN__)
+# define WIN32_LEAN_AND_MEAN
+# include <windows.h>
+#endif
 
 /*=========================================================================
  * String functions
@@ -33,11 +30,11 @@
  * @param number the 64-bit number to output
  * @param min_width the minimum width, the number must take
  */
-void sprintI64(char *dst, uint64_t number, int min_width)
+void sprintI64(char* dst, uint64_t number, int min_width)
 {
 	char buf[24]; /* internal buffer to output the number to */
 	size_t len;
-	char *p = buf + 23; /* start filling from the buffer end */
+	char* p = buf + 23; /* start filling from the buffer end */
 	*(p--) = 0; /* last symbol should be '\0' */
 	if (number == 0) {
 		*(p--) = '0';
@@ -51,7 +48,7 @@ void sprintI64(char *dst, uint64_t number, int min_width)
 		memset(dst, 0x20, min_width - len); /* fill by spaces */
 		dst += min_width - len;
 	}
-	memcpy(dst, p+1, len+1); /* copy the number to the output buffer */
+	memcpy(dst, p + 1, len + 1); /* copy the number to the output buffer */
 }
 
 /**
@@ -67,57 +64,6 @@ int int_len(uint64_t num)
 	return (len == 0 ? 1 : len); /* note: int_len(0) == 1 */
 }
 
-/**
- * Convert a byte to a hexadecimal string. The result, consisting of two
- * hexadecimal digits is stored into a buffer.
- *
- * @param dst  the buffer to receive two symbols of hex representation
- * @param byte the byte to decode
- * @param upper_case flag to print string in uppercase
- * @return pointer to the next char in buffer (dst+2)
- */
-static char* print_hex_byte(char *dst, const unsigned char byte, int upper_case)
-{
-	const char add = (upper_case ? 'A' - 10 : 'a' - 10);
-	unsigned char c = (byte >> 4) & 15;
-	*dst++ = (c > 9 ? c + add : c + '0');
-	c = byte & 15;
-	*dst++ = (c > 9 ? c + add : c + '0');
-	return dst;
-}
-
-/* unsafe characters are "<>{}[]%#/|\^~`@:;?=&+ */
-#define IS_GOOD_URL_CHAR(c) (isalnum((unsigned char)c) || strchr("$-_.!'(),", c))
-
-/**
- * URL-encode a string.
- *
- * @param dst buffer to receive result or NULL to calculate
- *    the lengths of encoded string
- * @param filename the file name
- * @return the length of the result string
- */
-int urlencode(char *dst, const char *name)
-{
-	const char *start;
-	if (!dst) {
-		int len;
-		for (len = 0; *name; name++) len += (IS_GOOD_URL_CHAR(*name) ? 1 : 3);
-		return len;
-	}
-	/* encode URL as specified by RFC 1738 */
-	for (start = dst; *name; name++) {
-		if ( IS_GOOD_URL_CHAR(*name) ) {
-			*dst++ = *name;
-		} else {
-			*dst++ = '%';
-			dst = print_hex_byte(dst, *name, 'A');
-		}
-	}
-	*dst = 0;
-	return (int)(dst - start);
-}
-
 /**
  * Convert given string to lower case.
  * The result string will be allocated by malloc.
@@ -166,23 +112,73 @@ char* str_set(char* buf, int ch, int length)
 	return buf;
 }
 
+#ifdef _WIN32
 /**
- * Concatenates two strings and returns allocated buffer with result.
+ * Return wide-string obtained from the given source string by replacing its part with another string.
  *
- * @param orig original string
- * @param append the string to append
- * @return the buffer
+ * @param src source wide-string
+ * @param start_pos starting position of the replacement
+ * @param end_pos ending position of the replacement
+ * @param replacement the replacement ASCII string (nullable), NULL is interpreted as empty string
+ * @return result of replacement, the resulting string is always allocated by malloc(),
+ *         and must be freed by caller
  */
-char* str_append(const char* orig, const char* append)
+wchar_t* wcs_replace_n(const wchar_t* src, size_t start_pos, size_t end_pos, const char* replacement)
 {
-	size_t len1 = strlen(orig);
-	size_t len2 = strlen(append);
-	char* res = (char*)rsh_malloc(len1 + len2 + 1);
+	const size_t len1 = wcslen(src);
+	const size_t len2 = (replacement ? strlen(replacement) : 0);
+	size_t result_len;
+	size_t i;
+	wchar_t* result;
+	if (start_pos > len1)
+		start_pos = end_pos = len1;
+	else if (end_pos > len1)
+		end_pos = len1;
+	else if (start_pos > end_pos)
+		end_pos = start_pos;
+	result_len = len1 + len2 - (end_pos - start_pos);
+	result = (wchar_t*)rsh_malloc((result_len + 1) * sizeof(wchar_t));
+	memcpy(result, src, start_pos * sizeof(wchar_t));
+	for (i = 0; i < len2; i++)
+		result[start_pos + i] = (wchar_t)replacement[i];
+	if (end_pos < len1)
+		memcpy(result + start_pos + len2, src + end_pos, (len1 - end_pos) * sizeof(wchar_t));
+	result[result_len] = 0;
+	return result;
+}
+#endif
 
-	/* concatenate two strings */
-	memcpy(res, orig, len1);
-	memcpy(res + len1, append, len2 + 1);
-	return res;
+/**
+ * Return string obtained from the given source string by replacing its part with another string.
+ *
+ * @param src source string
+ * @param start_pos starting position of the replacement
+ * @param end_pos ending position of the replacement
+ * @param replacement the replacement string (nullable), NULL is interpreted as empty string
+ * @return result of replacement, the resulting string is always allocated by malloc(),
+ *         and must be freed by caller
+ */
+char* str_replace_n(const char* src, size_t start_pos, size_t end_pos, const char* replacement)
+{
+	const size_t len1 = strlen(src);
+	const size_t len2 = (replacement ? strlen(replacement) : 0);
+	size_t result_len;
+	char* result;
+	if (start_pos > len1)
+		start_pos = end_pos = len1;
+	else if (end_pos > len1)
+		end_pos = len1;
+	else if (start_pos > end_pos)
+		end_pos = start_pos;
+	result_len = len1 + len2 - (end_pos - start_pos);
+	result = (char*)rsh_malloc(result_len + 1);
+	memcpy(result, src, start_pos);
+	if (len2 > 0)
+		memcpy(result + start_pos, replacement, len2);
+	if (end_pos < len1)
+		memcpy(result + start_pos + len2, src + end_pos, len1 - end_pos);
+	result[result_len] = 0;
+	return result;
 }
 
 /**
@@ -208,7 +204,7 @@ int is_binary_string(const char* str)
  * @param str the string to measure
  * @return number of utf8 characters in the string
  */
-size_t strlen_utf8_c(const char *str)
+size_t count_utf8_symbols(const char* str)
 {
 	size_t length = 0;
 	for (; *str; str++) {
@@ -218,8 +214,8 @@ size_t strlen_utf8_c(const char *str)
 }
 
 /*=========================================================================
-* Program version information
-*=========================================================================*/
+ * Program version information
+ *=========================================================================*/
 
 const char* get_version_string(void)
 {
@@ -233,134 +229,11 @@ const char* get_bt_program_name(void)
 	return bt_program_name;
 }
 
-/*=========================================================================
- * Path functions
- *=========================================================================*/
-
-/**
- * Return filename without path.
- *
- * @param path file path
- * @return filename
- */
-const char* get_basename(const char* path)
-{
-	const char *p = path + strlen(path) - 1;
-	for (; p >= path && !IS_PATH_SEPARATOR(*p); p--);
-	return (p+1);
-}
-
-/**
- * Return allocated buffer with the directory part of the path.
- * The buffer must be freed by calling free().
- *
- * @param path file path
- * @return directory
- */
-char* get_dirname(const char* path)
-{
-	const char *p = path + strlen(path) - 1;
-	char *res;
-	for (; p > path && !IS_PATH_SEPARATOR(*p); p--);
-	if ((p - path) > 1) {
-		res = (char*)rsh_malloc(p-path+1);
-		memcpy(res, path, p-path);
-		res[p-path] = 0;
-		return res;
-	} else {
-		return rsh_strdup(".");
-	}
-}
-
-/**
- * Assemble a filepath from its directory and filename.
- *
- * @param dir_path directory path
- * @param filename file name
- * @return filepath
- */
-char* make_path(const char* dir_path, const char* filename)
-{
-	char* buf;
-	size_t len;
-	assert(dir_path);
-	assert(filename);
-
-	/* remove leading path separators from filename */
-	while (IS_PATH_SEPARATOR(*filename)) filename++;
-
-	if (dir_path[0] == '.' && dir_path[1] == 0) {
-		/* do not extend filename for dir_path="." */
-		return rsh_strdup(filename);
-	}
-
-	/* copy directory path */
-	len = strlen(dir_path);
-	buf = (char*)rsh_malloc(len + strlen(filename) + 2);
-	strcpy(buf, dir_path);
-
-	/* separate directory from filename */
-	if (len > 0 && !IS_PATH_SEPARATOR(buf[len-1])) {
-		buf[len++] = SYS_PATH_SEPARATOR;
-	}
-
-	/* append filename */
-	strcpy(buf+len, filename);
-	return buf;
-}
-
-#define IS_ANY_SLASH(c) ((c) == RSH_T('/') || (c) == RSH_T('\\'))
-
-/**
- * Compare paths.
- *
- * @param a the first path
- * @param b the second path
- */
-int are_paths_equal(const rsh_tchar* a, const rsh_tchar* b)
-{
-	if (!a || !b) return 0;
-	if (a[0] == RSH_T('.') && IS_ANY_SLASH(a[1])) a += 2;
-	if (b[0] == RSH_T('.') && IS_ANY_SLASH(b[1])) b += 2;
-	
-	for (; *a; ++a, ++b)
-	{
-		if (*a != *b && (!IS_ANY_SLASH(*b) || !IS_ANY_SLASH(*a)))
-		{
-			/* paths are different */
-			return 0;
-		}
-		
-	}
-	/* check if both paths terminated */
-	return (*a == *b);
-}
-
 /*=========================================================================
  * Timer functions
  *=========================================================================*/
 
-/**
- * Return real-value representing number of seconds
- * stored in the given timeval structure.
- * The function is used with timers, when printing time statistics.
- *
- * @param delta time delta to be converted
- * @return number of seconds
- */
-static double rsh_fsec(timedelta_t* timer)
-{
-#ifdef _WIN32
-	LARGE_INTEGER freq;
-	QueryPerformanceFrequency(&freq);
-	return (double)*timer / freq.QuadPart;
-#else
-	return ((double)timer->tv_usec / 1000000.0) + timer->tv_sec;
-#endif
-}
-
-#ifdef _WIN32
-#include <windows.h>
+#if defined( _WIN32) || defined(__CYGWIN__)
 #define get_timedelta(delta) QueryPerformanceCounter((LARGE_INTEGER*)delta)
 #else
 #define get_timedelta(delta) gettimeofday(delta, NULL)
@@ -371,22 +244,26 @@ void rsh_timer_start(timedelta_t* timer)
 	get_timedelta(timer);
 }
 
-double rsh_timer_stop(timedelta_t* timer)
+uint64_t rsh_timer_stop(timedelta_t* timer)
 {
 	timedelta_t end;
+#if defined( _WIN32) || defined(__CYGWIN__)
+	LARGE_INTEGER frequency;
 	get_timedelta(&end);
-#ifdef _WIN32
 	*timer = end - *timer;
+	QueryPerformanceFrequency(&frequency);
+	return (uint64_t)((LONGLONG)(*timer) * 1000 / frequency.QuadPart);
 #else
+	get_timedelta(&end);
 	timer->tv_sec  = end.tv_sec  - timer->tv_sec - (end.tv_usec >= timer->tv_usec ? 0 : 1);
 	timer->tv_usec = end.tv_usec + (end.tv_usec >= timer->tv_usec ? 0 : 1000000 ) - timer->tv_usec;
+	return ((uint64_t)(timer->tv_sec) * 1000 + timer->tv_usec / 1000);
 #endif
-	return rsh_fsec(timer);
 }
 
 unsigned rhash_get_ticks(void)
 {
-#ifdef _WIN32
+#if defined( _WIN32) || defined(__CYGWIN__)
 	return GetTickCount();
 #else
 	struct timeval tv;
@@ -395,207 +272,65 @@ unsigned rhash_get_ticks(void)
 #endif
 }
 
-/*=========================================================================
- * File functions
- *=========================================================================*/
-
-void rsh_file_init(file_t* file, const char* path, int reuse_path)
-{
-	memset(file, 0, sizeof(*file));
-	if (reuse_path)
-	{
-		file->path = (char*)path;
-		file->mode = FILE_OPT_DONT_FREE_PATH;
-	} else {
-		file->path = rsh_strdup(path);
-	}
-}
-
-void rsh_file_cleanup(file_t* file)
-{
-	if ((file->mode & FILE_OPT_DONT_FREE_PATH) == 0)
-	{
-		free(file->path);
-	}
-	file->path = NULL;
-
-#ifdef _WIN32
-	free(file->wpath);
-	file->wpath = NULL;
-#endif /* _WIN32 */
-
-	file->mtime = file->size = 0;
-	file->mode = 0;
-}
-
-#ifdef _WIN32
-/**
- * Fill file information in the file_t structure.
- *
- * @param file the file information
- * @return 0 on success, -1 on error
- */
-int rsh_file_statw(file_t* file)
-{
-	WIN32_FILE_ATTRIBUTE_DATA data;
-
-	/* read file attributes */
-	if (GetFileAttributesExW(file->wpath, GetFileExInfoStandard, &data)) {
-		uint64_t u;
-		file->size  = (((uint64_t)data.nFileSizeHigh) << 32) + data.nFileSizeLow;
-		file->mode &= FILE_OPT_DONT_FREE_PATH;
-		file->mode |= (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? FILE_IFDIR : FILE_IFREG);
-
-		/* the number of 100-nanosecond intervals since January 1, 1601 */
-		u = (((uint64_t)data.ftLastWriteTime.dwHighDateTime) << 32) + data.ftLastWriteTime.dwLowDateTime;
-		/* convert to second and subtract the epoch difference */
-		file->mtime = u / 10000000 - 11644473600LL;
-		return 0;
-	}
-	set_errno_from_last_file_error();
-	return -1;
-}
-#endif
-
-/**
- * Fill file information in the file_t structure.
- *
- * @param file the file information
- * @param use_lstat nonzero if lstat() shall be used
- * @return 0 on success, -1 on error
- */
-int rsh_file_stat2(file_t* file, int use_lstat)
-{
-#ifdef _WIN32
-	int i;
-	(void)use_lstat; /* ignore on windows */
-
-	file->mtime = 0;
-	if (file->wpath) {
-		free(file->wpath);
-		file->wpath = NULL;
-	}
-
-	for (i = 0; i < 2; i++) {
-		file->wpath = c2w(file->path, i);
-		if (file->wpath == NULL) continue;
-
-		/* return on success */
-		if (rsh_file_statw(file) == 0) return 0;
-
-		free(file->wpath);
-		file->wpath = NULL;
-	}
-	/* NB: usually errno is set by the rsh_file_statw() call */
-	if (!errno) errno = EINVAL;
-	return -1;
-#else
-	struct stat st;
-	int res = 0;
-	file->mode &= FILE_OPT_DONT_FREE_PATH;
-
-	do {
-		if (use_lstat) {
-			/* check for symlink */
-			if (lstat(file->path, &st) < 0) return -1;
-			if (!S_ISLNK(st.st_mode)) break;
-
-			/* it's a symlink */
-			file->mode |= FILE_IFLNK;
-		}
-
-		res = stat(file->path, &st);
-	} while (0);
-
-	file->size  = st.st_size;
-	file->mtime = st.st_mtime;
-
-	if (S_ISDIR(st.st_mode)) {
-		file->mode |= FILE_IFDIR;
-	} else if (S_ISREG(st.st_mode)) {
-		/* it's a regular file or a symlink pointing to a regular file */
-		file->mode |= FILE_IFREG;
-	}
-
-	return res;
-#endif /* _WIN32 */
-}
-
-/**
- * Read the file information like its type, size and modification date.
- *
- * @param file the file information
- * @return 0 on success, -1 on error
- */
-int rsh_file_stat(file_t* file)
-{
-	return rsh_file_stat2(file, 0);
-}
-
-int is_regular_file(const char* path)
-{
-	int is_regular = 0;
-	file_t file;
-
-	rsh_file_init(&file, path, 1);
-	if (rsh_file_stat(&file) >= 0) {
-		is_regular = FILE_ISREG(&file);
-	}
-	rsh_file_cleanup(&file);
-	return is_regular;
-}
-
-int if_file_exists(const char* path)
-{
-	int exists;
-	file_t file;
-
-	rsh_file_init(&file, path, 1);
-	exists = (rsh_file_stat(&file) >= 0);
-	rsh_file_cleanup(&file);
-	return exists;
-}
-
 /*=========================================================================
  * Custom program exit function
  *=========================================================================*/
 
-/**
-* Exit the program, with restoring console state.
-*
-* @param code the program exit code
-*/
-void rhash_exit(int code)
+struct rhash_exit_handlers_t
 {
-	IF_WINDOWS(restore_console());
-	exit(code);
+	unsigned handlers_count;
+	exit_handler_t handlers[4];
+} rhash_exit_handlers = { 0, { 0 } };
+
+/**
+ * Install a handler to be called on program exit.
+ *
+ * @param handler the hadler to add
+ */
+void rsh_install_exit_handler(exit_handler_t handler)
+{
+	if (rhash_exit_handlers.handlers_count >= (sizeof(rhash_exit_handlers.handlers) / sizeof(rhash_exit_handlers.handlers[0])))
+	{
+		assert(!"to many handlers");
+		rsh_exit(2);
+	}
+	rhash_exit_handlers.handlers[rhash_exit_handlers.handlers_count] = handler;
+	rhash_exit_handlers.handlers_count++;
 }
 
-/*=========================================================================
- * Error reporting functions
- *=========================================================================*/
-
-static void report_error_default(const char* srcfile, int srcline,
-	const char* format, ...);
-
-void (*rsh_exit)(int code) = exit;
-void (*rsh_report_error)(const char* srcfile, int srcline,
-	const char* format, ...) = report_error_default;
+/**
+ * Remove the last installed exit handler.
+ */
+void rsh_remove_exit_handler(void)
+{
+	if (rhash_exit_handlers.handlers_count == 0)
+	{
+		assert(rhash_exit_handlers.handlers_count > 0 && "no handlers installed");
+		rsh_exit(2);
+	}
+	rhash_exit_handlers.handlers_count--;
+}
 
 /**
- * Print given library failure to stderr.
+ * Call all installed exit handlers, starting from the latest one.
  *
- * @param srcfile source file to report error on fail
- * @param srcline source code line to be reported on fail
- * @param format printf-formated error message
+ * @param code the program exit code
  */
-static void report_error_default(const char* srcfile, int srcline, const char* format, ...)
+void rsh_call_exit_handlers(void)
 {
-	va_list ap;
-	fprintf(stderr, "RHash: error at %s:%u: ", srcfile, srcline);
-	va_start(ap, format);
-	vfprintf(stderr, format, ap); /* report the error to stderr */
-	va_end(ap);
+	while (rhash_exit_handlers.handlers_count > 0)
+		rhash_exit_handlers.handlers[--rhash_exit_handlers.handlers_count]();
+}
+
+/**
+ * Call all installed exit handlers and exit the program.
+ *
+ * @param code the program exit code
+ */
+void rsh_exit(int code)
+{
+	rsh_call_exit_handlers();
+	exit(code);
 }
 
 /*=========================================================================
@@ -613,10 +348,7 @@ static void report_error_default(const char* srcfile, int srcline, const char* f
 void* rhash_malloc(size_t size, const char* srcfile, int srcline)
 {
 	void* res = malloc(size);
-	if (!res) {
-		rsh_report_error(srcfile, srcline, "%s(%u) failed\n", "malloc", (unsigned)size);
-		rsh_exit(2);
-	}
+	RSH_REQUIRE_IMPL(res, srcfile, srcline, "%malloc(%u) failed\n", (unsigned)size);
 	return res;
 }
 
@@ -632,10 +364,7 @@ void* rhash_malloc(size_t size, const char* srcfile, int srcline)
 void* rhash_calloc(size_t num, size_t size, const char* srcfile, int srcline)
 {
 	void* res = calloc(num, size);
-	if (!res) {
-		rsh_report_error(srcfile, srcline, "calloc(%u, %u) failed\n", (unsigned)num, (unsigned)size);
-		rsh_exit(2);
-	}
+	RSH_REQUIRE_IMPL(res, srcfile, srcline, "calloc(%u, %u) failed\n", (unsigned)num, (unsigned)size);
 	return res;
 }
 
@@ -656,11 +385,7 @@ char* rhash_strdup(const char* str, const char* srcfile, int srcline)
 	char* res = (char*)malloc(strlen(str)+1);
 	if (res) strcpy(res, str);
 #endif
-
-	if (!res) {
-		rsh_report_error(srcfile, srcline, "strdup(\"%s\") failed\n", str);
-		rsh_exit(2);
-	}
+	RSH_REQUIRE_IMPL(res, srcfile, srcline, "strdup(\"%s\") failed\n", str);
 	return res;
 }
 
@@ -681,11 +406,7 @@ wchar_t* rhash_wcsdup(const wchar_t* str, const char* srcfile, int srcline)
 	wchar_t* res = (wchar_t*)malloc((wcslen(str) + 1) * sizeof(wchar_t));
 	if (res) wcscpy(res, str);
 #endif
-
-	if (!res) {
-		rsh_report_error(srcfile, srcline, "wcsdup(\"%u\") failed\n", (wcslen(str) + 1));
-		rsh_exit(2);
-	}
+	RSH_REQUIRE_IMPL(res, srcfile, srcline, "wcsdup(\"%u\") failed\n", (wcslen(str) + 1));
 	return res;
 }
 #endif
@@ -702,13 +423,59 @@ wchar_t* rhash_wcsdup(const wchar_t* str, const char* srcfile, int srcline)
 void* rhash_realloc(void* mem, size_t size, const char* srcfile, int srcline)
 {
 	void* res = realloc(mem, size);
-	if (!res) {
-		rsh_report_error(srcfile, srcline, "realloc(%p, %u) failed\n", mem, (unsigned)size);
-		rsh_exit(2);
-	}
+	RSH_REQUIRE_IMPL(res, srcfile, srcline, "realloc(%p, %u) failed\n", mem, (unsigned)size);
 	return res;
 }
 
+/*=========================================================================
+ * Bit functions
+ *=========================================================================*/
+ #ifndef get_ctz
+ /**
+  * Returns index of the least significant set bit in a 32-bit number.
+  * This operation is also known as Count Trailing Zeros (CTZ).
+  *
+  * The function is a portable, branch-free equivalent of GCC's __builtin_ctz(),
+  * using a De Bruijn sequence for constant-time lookup.
+  *
+  * @param x 32-bit unsigned integer to analyze (must not be zero)
+  * @return zero-based index of the least significant set bit (0 to 31)
+ */
+unsigned get_ctz(unsigned x)
+{
+	/* array for conversion to bit position */
+	static unsigned bit_pos[32] =  {
+		0, 1, 28, 2, 29, 14, 24, 3, 30, 22, 20, 15, 25, 17, 4, 8,
+		31, 27, 13, 23, 21, 19, 16, 7, 26, 12, 18, 6, 11, 5, 10, 9
+	};
+	return bit_pos[((uint32_t)((x & -x) * 0x077CB531U)) >> 27];
+}
+/**
+ * Returns the zero-based index of the least significant set bit in a 64-bit number.
+ * This operation is also known as Count Trailing Zeros (CTZ).
+ *
+ * The function is a portable, branch-free equivalent of GCC's __builtin_ctzll().
+ * Uses a 32-bit optimized implementation with magic constant `0x78291ACF`,
+ * based on Matt Taylor's original algorithm (2003).
+ *
+ * @param x 64-bit unsigned integer to analyze (must not be zero)
+ * @return zero-based index of the least significant set bit (0 to 63)
+ */
+unsigned get_ctz64(uint64_t x)
+{
+	/* lookup table mapping hash values to bit position */
+	static const unsigned bit_pos[64] =  {
+		63, 30,  3, 32, 59, 14, 11, 33, 60, 24, 50,  9, 55, 19, 21, 34,
+		61, 29,  2, 53, 51, 23, 41, 18, 56, 28,  1, 43, 46, 27,  0, 35,
+		62, 31, 58,  4,  5, 49, 54,  6, 15, 52, 12, 40,  7, 42, 45, 16,
+		25, 57, 48, 13, 10, 39,  8, 44, 20, 47, 38, 22, 17, 37, 36, 26
+	};
+	x ^= x - 1;
+	uint32_t folded = (uint32_t)((x >> 32) ^ x);
+	return bit_pos[folded * 0x78291ACF >> 26];
+}
+#endif
+
 /*=========================================================================
  * Containers
  *=========================================================================*/
@@ -750,7 +517,7 @@ void rsh_vector_destroy(vector_t* vect)
 	if (!vect) return;
 	if (vect->destructor) {
 		unsigned i;
-		for (i=0; i<vect->size; i++) vect->destructor(vect->array[i]);
+		for (i = 0; i < vect->size; i++) vect->destructor(vect->array[i]);
 	}
 	free(vect->array);
 	vect->size = vect->allocated = 0;
@@ -778,7 +545,7 @@ void rsh_vector_add_ptr(vector_t* vect, void* item)
 {
 	/* check if vect contains enough space for the next item */
 	if (vect->size >= vect->allocated) {
-		size_t size = (vect->allocated==0 ? 128 : vect->allocated * 2);
+		size_t size = (vect->allocated == 0 ? 128 : vect->allocated * 2);
 		vect->array = (void**)rsh_realloc(vect->array, size * sizeof(void*));
 		vect->allocated = size;
 	}
@@ -797,7 +564,7 @@ void rsh_vector_add_empty(struct vector_t* vect, size_t item_size)
 {
 	/* check if vect contains enough space for next item */
 	if (vect->size >= vect->allocated) {
-		size_t size = (vect->allocated==0 ? 128 : vect->allocated * 2);
+		size_t size = (vect->allocated == 0 ? 128 : vect->allocated * 2);
 		vect->array = (void**)rsh_realloc(vect->array, size * item_size);
 		vect->allocated = size;
 	}
@@ -837,7 +604,7 @@ void rsh_blocks_vector_destroy(blocks_vector_t* bvector)
  */
 strbuf_t* rsh_str_new(void)
 {
-	strbuf_t *res = (strbuf_t*)malloc(sizeof(strbuf_t));
+	strbuf_t* res = (strbuf_t*)rsh_malloc(sizeof(strbuf_t));
 	memset(res, 0, sizeof(strbuf_t));
 	return res;
 }
@@ -862,7 +629,7 @@ void rsh_str_free(strbuf_t* ptr)
  * @param str pointer to the string-buffer object
  * @param new_size number of bytes buffer must contain
  */
-void rsh_str_ensure_size(strbuf_t *str, size_t new_size)
+void rsh_str_ensure_size(strbuf_t* str, size_t new_size)
 {
 	if (new_size >= (size_t)str->allocated) {
 		if (new_size < 64) new_size = 64;
@@ -879,10 +646,11 @@ void rsh_str_ensure_size(strbuf_t *str, size_t new_size)
  *
  * @param str pointer to the string buffer
  * @param text the text to append
- * @param length number of character to append.
+ * @param length number of character to append
  */
-void rsh_str_append_n(strbuf_t *str, const char* text, size_t length)
+void rsh_str_append_n(strbuf_t* str, const char* text, size_t length)
 {
+	assert(text);
 	rsh_str_ensure_length(str, str->len + length + 1);
 	memcpy(str->str + str->len, text, length);
 	str->len += length;
@@ -895,7 +663,8 @@ void rsh_str_append_n(strbuf_t *str, const char* text, size_t length)
  * @param str pointer to the string buffer
  * @param text the null-terminated string to append
  */
-void rsh_str_append(strbuf_t *str, const char* text)
+void rsh_str_append(strbuf_t* str, const char* text)
 {
+	assert(text);
 	rsh_str_append_n(str, text, strlen(text));
 }
