@@ -1,56 +1,52 @@
-/* common_func.h */
+/* common_func.h - commonly used functions */
 #ifndef COMMON_FUNC_H
 #define COMMON_FUNC_H
 
-/* use 64-bit off_t.
- * these macros must be defined before any include file */
-#define _LARGEFILE64_SOURCE
-#define _FILE_OFFSET_BITS 64
-
 /* internationalization support via gettext/libintl */
 #ifdef USE_GETTEXT
 # include <libintl.h>
 # define _(str) gettext(str)
-# ifdef _WIN32
-#  define LOCALEDIR "./locale"
-# else /* _WIN32 */
+# define TEXT_DOMAIN "rhash"
+# ifndef LOCALEDIR
 #  define LOCALEDIR "/usr/share/locale"
-# endif /* _WIN32 */
+# endif /* LOCALEDIR */
 #else
 # define _(str) (str)
 #endif /* USE_GETTEXT */
 
+#include <stddef.h> /* for wchar_t */
 #include <stdint.h>
 #include <stdio.h>
 #include <time.h> /* for time_t */
-#include <stddef.h> /* for wchar_t */
-
-#ifndef _WIN32
-#include <sys/time.h> /* for timeval */
-/*#else
-#include <windows.h>*/
+#if !defined( _WIN32) && !defined(__CYGWIN__)
+# include <sys/time.h> /* for timeval */
 #elif _MSC_VER > 1300
-#include "win32/platform-dependent.h"
+# include "platform.h"
 #endif
 
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 /* string function */
-void sprintI64(char *dst, uint64_t number, int max_width);
+void sprintI64(char* dst, uint64_t number, int max_width);
 int  int_len(uint64_t num);
 
-int  urlencode(char *dst, const char *name);
+int  urlencode(char* dst, const char* name);
+size_t count_utf8_symbols(const char* str);
 int  is_binary_string(const char* str);
 char* str_tolower(const char* str);
 char* str_trim(char* str);
 char* str_set(char* buf, int ch, int size);
-char* str_append(const char* orig, const char* append);
-size_t strlen_utf8_c(const char *str);
+char* str_replace_n(const char* src, size_t start_pos, size_t end_pos, const char* replace);
+#ifdef _WIN32
+wchar_t* wcs_replace_n(const wchar_t* src, size_t start_pos, size_t end_pos, const char* replace);
+#endif /* _WIN32 */
 
+/* check if character starts a commentary in the program config file */
 #define IS_COMMENT(c) ((c) == ';' || (c) == '#')
+/* test line for UTF8 mark: 0xEF, 0xBB, 0xBF */
+#define STARTS_WITH_UTF8_BOM(line) ((line)[0] == '\357' && (line)[1] == '\273' && (line)[2] == '\277')
 
 #ifdef _WIN32
 typedef wchar_t rsh_tchar;
@@ -59,80 +55,38 @@ typedef wchar_t rsh_tchar;
 typedef  char rsh_tchar;
 # define RSH_T(str) str
 #endif /* _WIN32 */
+typedef rsh_tchar* tstr_t;
+typedef const rsh_tchar* ctstr_t;
 
 #ifdef _WIN32
 # define IF_WINDOWS(code) code
-# define SYS_PATH_SEPARATOR '\\'
-# define IS_PATH_SEPARATOR(c) ((c) == '\\' || (c) == '/')
-# define IS_PATH_SEPARATOR_W(c) ((c) == L'\\' || (c) == L'/')
-# define is_utf8() win_is_utf8()
-# define to_utf8(str) win_to_utf8(str)
 #else /* non _WIN32 part */
 # define IF_WINDOWS(code)
-# define SYS_PATH_SEPARATOR '/'
-# define IS_PATH_SEPARATOR(c) ((c) == '/')
-/* stub for utf8 */
-# define is_utf8() 1
-# define to_utf8(str) NULL
 #endif /* _WIN32 */
 
 /* version information */
 const char* get_version_string(void);
 const char* get_bt_program_name(void);
 
-
-/**
- * Portable file information.
- */
-typedef struct file_t
-{
-	char* path;
-	wchar_t* wpath;
-	uint64_t size;
-	uint64_t mtime;
-	unsigned mode;
-} file_t;
-
-/* bit constants for the file_t.mode bit mask */
-#define FILE_IFDIR   0x01
-#define FILE_IFLNK   0x02
-#define FILE_IFREG   0x04
-#define FILE_IFROOT  0x10
-#define FILE_IFSTDIN 0x20
-#define FILE_OPT_DONT_FREE_PATH  0x40
-
-#define FILE_ISDIR(file) ((file)->mode & FILE_IFDIR)
-#define FILE_ISLNK(file) ((file)->mode & FILE_IFLNK)
-#define FILE_ISREG(file) ((file)->mode & FILE_IFREG)
-
-/* file functions */
-
-const char* get_basename(const char* path);
-char* get_dirname(const char* path);
-char* make_path(const char* dir, const char* filename);
-int are_paths_equal(const rsh_tchar* a, const rsh_tchar* b);
-
-void rsh_file_init(file_t* file, const char* path, int reuse_path);
-void rsh_file_cleanup(file_t* file);
-int rsh_file_stat(file_t* file);
-int rsh_file_stat2(file_t* file, int use_lstat);
-int is_regular_file(const char* path);
-int if_file_exists(const char* path);
-
+/* printf functions */
 #ifdef _WIN32
-# define get_file_tpath(file) ((file)->wpath)
-int rsh_file_statw(file_t* file);
-# define rsh_fopen_bin(path, mode) win_fopen_bin(path, mode)
+# define rsh_fprintf win_fprintf
+# define rsh_fprintf_targ win_fprintf_warg
+# define rsh_vfprintf win_vfprintf
+# define rsh_fwrite win_fwrite
 #else
-# define get_file_tpath(file) ((file)->path)
-# define rsh_fopen_bin(path, mode) fopen(path, mode)
+# define rsh_fprintf fprintf
+# define rsh_fprintf_targ fprintf
+# define rsh_vfprintf vfprintf
+# define rsh_fwrite fwrite
 #endif
 
+#define PRINTF_RES(res) ((res) < 0 ? res : 0)
 
-/* time data and functions */
+/* time delta and timer functions */
 
 /* portable timer definition */
-#ifdef _WIN32
+#if defined( _WIN32) || defined(__CYGWIN__)
 typedef unsigned long long timedelta_t;
 #else
 #include <sys/time.h> /* for timeval */
@@ -150,22 +104,25 @@ void rsh_timer_start(timedelta_t* timer);
  * Stop given timer.
  *
  * @param timer the timer to stop
- * @return number of seconds timed
+ * @return timer value in milliseconds
  */
-double rsh_timer_stop(timedelta_t* timer);
+uint64_t rsh_timer_stop(timedelta_t* timer);
 
 /**
  * Return ticks in milliseconds for time intervals measurement.
- * This function should be not precise but the fastest one
- * to retrieve internal clock value.
+ * This function should be optimized for speed and retrieve
+ * internal clock value, if possible.
  *
  * @return ticks count in milliseconds
  */
 unsigned rhash_get_ticks(void);
 
-
-void  rhash_exit(int code);
-
+/* program exit handlers */
+typedef void (*exit_handler_t)(void);
+void rsh_install_exit_handler(exit_handler_t handler);
+void rsh_remove_exit_handler(void);
+void rsh_call_exit_handlers(void);
+void rsh_exit(int code);
 
 /* clever malloc with error detection */
 #define rsh_malloc(size) rhash_malloc(size, __FILE__, __LINE__)
@@ -179,17 +136,31 @@ void* rhash_realloc(void* mem, size_t size, const char* srcfile, int srcline);
 
 #ifdef _WIN32
 #define rsh_wcsdup(str) rhash_wcsdup(str, __FILE__, __LINE__)
+#define rsh_tstrdup(str) rsh_wcsdup(str)
 wchar_t* rhash_wcsdup(const wchar_t* str, const char* srcfile, int srcline);
+#else
+#define rsh_tstrdup(str) rsh_strdup(str)
 #endif
 
+#ifndef __has_builtin
+# define __has_builtin(x) 0
+#endif
 
-extern void (*rsh_exit)(int code);
-extern void (*rsh_report_error)(const char* srcfile, int srcline, const char* format, ...);
+/* get_ctz - count traling zero bits */
+#if (defined(__GNUC__) && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))) || \
+    (defined(__clang__) && __has_builtin(__builtin_ctz))
+/* macro if GCC >= 3.4 or has clang __builtin_ctz() */
+# define get_ctz(x) __builtin_ctz(x)
+# define get_ctz64(x) __builtin_ctzll(x)
+#else
+unsigned get_ctz(unsigned);
+unsigned get_ctz64(uint64_t);
+#endif
 
 /* vector functions */
 typedef struct vector_t
 {
-	void **array;
+	void** array;
 	size_t size;
 	size_t allocated;
 	void (*destructor)(void*);
@@ -199,7 +170,7 @@ vector_t* rsh_vector_new(void (*destructor)(void*));
 vector_t* rsh_vector_new_simple(void);
 void rsh_vector_free(vector_t* vect);
 void rsh_vector_destroy(vector_t* vect);
-void rsh_vector_add_ptr(vector_t* vect, void *item);
+void rsh_vector_add_ptr(vector_t* vect, void* item);
 void rsh_vector_add_empty(vector_t* vect, size_t item_size);
 #define rsh_vector_add_uint32(vect, item) { \
 	rsh_vector_add_empty(vect, item_size); \
@@ -250,9 +221,9 @@ typedef struct strbuf_t
 
 strbuf_t* rsh_str_new(void);
 void rsh_str_free(strbuf_t* buf);
-void rsh_str_ensure_size(strbuf_t *str, size_t new_size);
-void rsh_str_append_n(strbuf_t *str, const char* text, size_t len);
-void rsh_str_append(strbuf_t *str, const char* text);
+void rsh_str_ensure_size(strbuf_t* str, size_t new_size);
+void rsh_str_append_n(strbuf_t* str, const char* text, size_t len);
+void rsh_str_append(strbuf_t* str, const char* text);
 
 #define rsh_str_ensure_length(str, len) \
 	if ((size_t)(len) >= (size_t)(str)->allocated) rsh_str_ensure_size((str), (len) + 1);
