@@ -1,7 +1,7 @@
 /* torrent.h */
 #ifndef TORRENT_H
 #define TORRENT_H
-#include "ustd.h"
+#include "algorithms.h"
 #include "sha1.h"
 
 #ifdef __cplusplus
@@ -13,7 +13,7 @@ extern "C" {
 /* vector structure */
 typedef struct torrent_vect
 {
-	void **array;     /* array of elements of the vector */
+	void** array;     /* array of elements of the vector */
 	size_t size;      /* vector size */
 	size_t allocated; /* number of allocated elements */
 } torrent_vect;
@@ -34,38 +34,47 @@ typedef struct torrent_ctx
 	sha1_ctx sha1_context;  /* context for hashing current file piece */
 #if defined(USE_OPENSSL) || defined(OPENSSL_RUNTIME)
 	unsigned long reserved; /* need more space for OpenSSL SHA1 context */
-	void *sha_init, *sha_update, *sha_final;
 #endif
 	size_t index;             /* byte index in the current piece */
 	size_t piece_length;      /* length of a torrent file piece */
 	size_t piece_count;       /* the number of pieces processed */
+	size_t error;             /* non-zero if error occurred, zero otherwise */
 	torrent_vect hash_blocks; /* array of blocks storing SHA1 hashes */
 	torrent_vect files;       /* names of files in a torrent batch */
+	torrent_vect announce;    /* announce URLs */
 	char* program_name;       /* the name of the program */
-	char* announce;           /* announce URL */
 
 	torrent_str content;      /* the content of generated torrent file */
-	int error; /* non-zero if error occurred, zero otherwise */
+#if defined(USE_OPENSSL) || defined(OPENSSL_RUNTIME)
+	rhash_hashing_methods sha1_methods;
+#endif
 } torrent_ctx;
 
-void bt_init(torrent_ctx *ctx);
-void bt_update(torrent_ctx *ctx, const void* msg, size_t size);
-void bt_final(torrent_ctx *ctx, unsigned char result[20]);
-void bt_cleanup(torrent_ctx *ctx);
+void bt_init(torrent_ctx* ctx);
+void bt_update(torrent_ctx* ctx, const void* msg, size_t size);
+void bt_final(torrent_ctx* ctx, unsigned char result[20]);
+void bt_cleanup(torrent_ctx* ctx);
 
-unsigned char* bt_get_btih(torrent_ctx *ctx);
-size_t bt_get_text(torrent_ctx *ctx, char** pstr);
+#if !defined(NO_IMPORT_EXPORT)
+size_t bt_export(const torrent_ctx* ctx, void* out, size_t size);
+size_t bt_import(torrent_ctx* ctx, const void* in, size_t size);
+#endif /* !defined(NO_IMPORT_EXPORT) */
+
+unsigned char* bt_get_btih(torrent_ctx* ctx);
+size_t bt_get_text(torrent_ctx* ctx, char** pstr);
 
 /* possible options */
 #define BT_OPT_PRIVATE 1
 #define BT_OPT_INFOHASH_ONLY 2
+#define BT_OPT_TRANSMISSION 4
 
-void bt_set_options(torrent_ctx *ctx, unsigned options);
-int  bt_add_file(torrent_ctx *ctx, const char* path, uint64_t filesize);
-int  bt_set_announce(torrent_ctx *ctx, const char* announce_url);
-int  bt_set_program_name(torrent_ctx *ctx, const char* name);
-void bt_set_piece_length(torrent_ctx *ctx, size_t piece_length);
-size_t bt_default_piece_length(uint64_t total_size);
+void bt_set_options(torrent_ctx* ctx, unsigned options);
+int  bt_add_file(torrent_ctx* ctx, const char* path, uint64_t filesize);
+int  bt_add_announce(torrent_ctx* ctx, const char* announce_url);
+int  bt_set_program_name(torrent_ctx* ctx, const char* name);
+void bt_set_piece_length(torrent_ctx* ctx, size_t piece_length);
+void bt_set_total_batch_size(torrent_ctx* ctx, uint64_t total_size);
+size_t bt_default_piece_length(uint64_t total_size, int transmission);
 
 #ifdef __cplusplus
 } /* extern "C" */
